#!/bin/sh
#   Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netinfo.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netinfo.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

# source common functions
. ./errlib.sh
. ./jsonlib.sh

# needed for opkg to function properly
export PATH=/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin:/sbin
export SHELL=/bin/sh

LOCKFILE=/var/lock/opkg.flock
LOGDIR="/var/log/dss11-websetup"
mkdir -p $LOGDIR 1>&2 >/dev/null

wait_for_busy_opkg() {
    flock -o -x $LOCKFILE -c "$1"
}

echo "Content-Type: application/json"
echo ""

if [[ ! -x /usr/bin/opkg ]]; then
    json_error $ERR_SETUP "opkg package manager is missing!"
    exit 1
fi

if [[ ! -x /usr/bin/addon ]]; then
    json_error $ERR_SETUP "addon wrapper script is missing!"
    exit 1
fi

if [[ "$1" == "list" ]]; then
    wait_for_busy_opkg "opkg2json -l"
elif [[ "$1" == "list-installed" ]]; then
    wait_for_busy_opkg "opkg2json -i"
elif [[ "$1" == "list-upgradable" ]]; then
    wait_for_busy_opkg "/www/scripts/rmaddons.sh"
    wait_for_busy_opkg "opkg2json -u"
elif [[ "$1" == "addon-list" ]]; then
    wait_for_busy_opkg "addon2json -l -t"
elif [[ "$1" == "addon-list-installed" ]]; then
    wait_for_busy_opkg "/www/scripts/rmaddons.sh"
    wait_for_busy_opkg "addon2json -i -t"
elif [[ "$1" == "addon-list-upgradable" ]]; then
    wait_for_busy_opkg "addon2json -u -t"
elif [[ "$1" == "addon-list-system" ]]; then
    wait_for_busy_opkg "opkg2json -i -t -f system-addon"
elif [[ "$1" == "update" ]] || [[ "$1" == "upgrade" ]] || 
     [[ "$1" == "addon-update" ]] || [[ "$1" == "addon-upgrade" ]] ||
     [[ "$1" == "addon-remove" ]] || [[ "$1" == "addon-install" ]]; then

    if [[ "$1" == "addon-install" ]]; then
        DISCLAIMER_ACCEPTED="no"
        if [[ -e /www/scripts/disclaimer.cfg ]]; then
            . /www/scripts/disclaimer.cfg
        fi
        if [[ "$DISCLAIMER_ACCEPTED"  != "yes" ]]; then
            json_error $ERR_DISCLAIMER "Please accept the disclaimer before installing apps"
            exit 1
        fi
    fi

    echo "[`date`] Starting update script (\"$1\")" --- >> "$LOGDIR/update.log"
    # We need to prevent that starting an update action here and requesting
    # the update status gets out of sync. This seems to happen with showupdate=1
    # once in a while, because a lot of different processes wait on the lock.
    # Since we start the update script in the background, we are not
    # blocked and as the result exit to the UI while the actual update
    # task is waiting for its turn in the background. The UI then requests
    # the update status, but gets an old status because the update task that
    # it triggered did not start yet (blocked, waiting for the lock).
    # So, what we will do is, we will call flock with the nonblock option,
    # evaluate the return code, and loop, thus blocking the current AJAX
    # call for as long as the update script is waiting for the lock.
    FLOCKCODE=1
    while [[ "$FLOCKCODE" -eq 1 ]]; do
        echo "[`date`] opkg.sh: flock'ing doupdate.sh" >> "$LOGDIR/update.log"
        flock -n -o -x $LOCKFILE -c "/www/scripts/doupdate.sh $1 $2" < /dev/null >> "$LOGDIR/update.log" &
        FLOCKCODE="$?"
        echo "[`date`] opkg.sh: flock returned $FLOCKCODE" >> "$LOGDIR/update.log"
        usleep 2000000
    done
    echo "[`date`] opkg.sh: flock loop done with code $FLOCKCODE" >> "$LOGDIR/update.log"
    if [[ "$FLOCKCODE" -eq "$ERR_OK" ]]; then
        json_ok "OK"
        exit 0
    elif [[ "$FLOCKCODE" -gt 1 ]]; then
        json_error $ERR_UPDATE "Failed to start package manager process!"
        exit 1
    fi
else 
    json_error $ERR_PARAM "Invalid option given!"
fi

